searchData={"items":[{"type":"module","title":"Styler","doc":"Styler is a formatter plugin with stronger opinions on code organization, multi-line defs and other code-style matters.","ref":"Styler.html"},{"type":"module","title":"Styler.Dealias","doc":"A datastructure for maintaining something like compiler alias state when traversing AST.\n\nNot anywhere as correct as what the compiler gives us, but close enough for open source work.","ref":"Styler.Dealias.html"},{"type":"function","title":"Styler.Dealias.apply/2","doc":"","ref":"Styler.Dealias.html#apply/2"},{"type":"function","title":"Styler.Dealias.new/1","doc":"","ref":"Styler.Dealias.html#new/1"},{"type":"function","title":"Styler.Dealias.put/2","doc":"","ref":"Styler.Dealias.html#put/2"},{"type":"behaviour","title":"Styler.Style","doc":"A Style takes AST and returns a transformed version of that AST.\n\nBecause these transformations involve traversing trees (the \"T\" in \"AST\"), we wrap the AST in a structure\ncalled a Zipper to facilitate walking the trees.","ref":"Styler.Style.html"},{"type":"function","title":"Styler.Style.displace_comments/2","doc":"Set the line of all comments with `line` in `range_start..range_end` to instead have line `range_start`","ref":"Styler.Style.html#displace_comments/2"},{"type":"function","title":"Styler.Style.ensure_block_parent/1","doc":"Returns the current node (wrapped in a `__block__` if necessary) if it's a valid place to insert additional nodes","ref":"Styler.Style.html#ensure_block_parent/1"},{"type":"function","title":"Styler.Style.find_nearest_block/1","doc":"Returns a zipper focused on the nearest node where additional nodes can be inserted (a \"block\").\n\nThe nearest node is either the current node, an ancestor, or one of those two but wrapped in a new `:__block__` node.","ref":"Styler.Style.html#find_nearest_block/1"},{"type":"function","title":"Styler.Style.fix_line_numbers/2","doc":"\"Fixes\" the line numbers of nodes who have had their orders changed via sorting or other methods.\nThis \"fix\" siply ensures that comments don't get wrecked as part of us moving AST nodes willy-nilly.\n\nThe fix is rather naive, and simply enforces the following property on the code:\nA given node must have a line number less than the following node.\nEt voila! Comments behave much better.","ref":"Styler.Style.html#fix_line_numbers/2"},{"type":"function","title":"In Detail - Styler.Style.fix_line_numbers/2","doc":"For example, given document\n\n  1: defmodule ...\n  2: alias B\n  3: # this is foo\n  4: def foo ...\n  5: alias A\n\nSorting aliases the ast node for  would put `alias A` (line 5) before `alias B` (line 2).\n\n  1: defmodule ...\n  5: alias A\n  2: alias B\n  3: # this is foo\n  4: def foo ...\n\nElixir's document algebra would then encounter `line: 5` and immediately dump all comments with `line <= 5`,\nmeaning after running through the formatter we'd end up with\n\n  1: defmodule\n  2: # hi\n  3: # this is foo\n  4: alias A\n  5: alias B\n  6:\n  7: def foo ...\n\nThis function fixes that by seeing that `alias A` has a higher line number than its following sibling `alias B` and so\nupdates `alias A`'s line to be preceding `alias B`'s line.\n\nRunning the results of this function through the formatter now no longer dumps the comments prematurely\n\n  1: defmodule ...\n  2: alias A\n  3: alias B\n  4: # this is foo\n  5: def foo ...","ref":"Styler.Style.html#fix_line_numbers/2-in-detail"},{"type":"function","title":"Styler.Style.max_line/1","doc":"","ref":"Styler.Style.html#max_line/1"},{"type":"function","title":"Styler.Style.reset_newlines/1","doc":"Takes a list of nodes and clumps them up, setting `end_of_expression: [newlines: x]` to 1 for all but the final node,\nwhich gets 2 instead, (hopefully!) creating an empty line before whatever follows.","ref":"Styler.Style.html#reset_newlines/1"},{"type":"function","title":"Styler.Style.reset_newlines/2","doc":"","ref":"Styler.Style.html#reset_newlines/2"},{"type":"callback","title":"Styler.Style.run/2","doc":"`run` will be used with `Zipper.traverse_while/3`, meaning it will be executed on every node of the AST.\n\nYou can skip traversing parts of the tree by returning a Zipper that's further along in the traversal, for example\nby calling `Zipper.skip(zipper)` to skip an entire subtree you know is of no interest to your Style.","ref":"Styler.Style.html#c:run/2"},{"type":"function","title":"Styler.Style.set_line/3","doc":"Recursively sets `:line` meta to `line`. Deletes `:newlines` unless `delete_lines: false` is passed","ref":"Styler.Style.html#set_line/3"},{"type":"function","title":"Styler.Style.shift_comments/2","doc":"Perform a series of shifts in a single pass.\n\nWhen shifting comments from block A to block B, naively using two passes of `shift_comments/3` would result\nin all comments ending up in either region A or region B (because A would move to B, then all B back to A)\nThis function exists to make sure that a comment is only moved once during the swap.","ref":"Styler.Style.html#shift_comments/2"},{"type":"function","title":"Styler.Style.shift_comments/3","doc":"Change the `line` of all comments with `line` in `range` by adding `delta` to it.\nA positive delta will move the lines further down a file, while a negative delta will move them up.","ref":"Styler.Style.html#shift_comments/3"},{"type":"function","title":"Styler.Style.shift_line/2","doc":"Recursively updates `:line` meta by adding `delta`","ref":"Styler.Style.html#shift_line/2"},{"type":"function","title":"Styler.Style.update_all_meta/2","doc":"Traverses an ast node, updating all nodes' meta with `meta_fun`","ref":"Styler.Style.html#update_all_meta/2"},{"type":"function","title":"Styler.Style.without_meta/1","doc":"","ref":"Styler.Style.html#without_meta/1"},{"type":"type","title":"Styler.Style.context/0","doc":"","ref":"Styler.Style.html#t:context/0"},{"type":"module","title":"Styler.Style.Blocks","doc":"Simple 1-1 rewrites all crammed into one module to make for more efficient traversals\n\nCredo Rules addressed:\n\n* Credo.Check.Consistency.ParameterPatternMatching\n* Credo.Check.Readability.LargeNumbers\n* Credo.Check.Readability.ParenthesesOnZeroArityDefs\n* Credo.Check.Readability.PreferImplicitTry\n* Credo.Check.Readability.WithSingleClause\n* Credo.Check.Refactor.CaseTrivialMatches\n* Credo.Check.Refactor.CondStatements\n* Credo.Check.Refactor.RedundantWithClauseResult\n* Credo.Check.Refactor.WithClauses","ref":"Styler.Style.Blocks.html"},{"type":"function","title":"Styler.Style.Blocks.run/2","doc":"","ref":"Styler.Style.Blocks.html#run/2"},{"type":"module","title":"Styler.Style.Configs","doc":"Orders `Config.config/2,3` stanzas in configuration files.\n\n- ordering is done only within immediate-sibling config statements\n- assignments are moved above the configuration blocks\n- any non `config/2,3` or assignment (`=/2`) calls mark the end of a sorting block.\n  this is support having conditional blocks (`if/case/cond`) and `import_config` stanzas between blocks\n\n#","ref":"Styler.Style.Configs.html"},{"type":"module","title":"Breakages - Styler.Style.Configs","doc":"If you configure the same values multiple times, Styler may swap their orders\n\n**Before**\n\n  line 04: config :foo, bar: :zab\n  line 40: config :foo, bar: :baz\n\n  # Application.fetch_env!(:foo)[:bar] => :baz\n\n**After**\n\n  line 04: config :foo, bar: :baz\n  line 05: config :foo, bar: :zab\n\n  # Application.fetch_env!(:foo)[:bar] => :zab\n\n**Fix**\n\nThe reason Styler sorts configuration is to help you noticed these duplicated configuration stanzas.\nDelete the duplicative/erroneous stanza and life will be good.","ref":"Styler.Style.Configs.html#module-breakages"},{"type":"function","title":"Styler.Style.Configs.run/2","doc":"","ref":"Styler.Style.Configs.html#run/2"},{"type":"function","title":"Styler.Style.Configs.set_lines/5","doc":"","ref":"Styler.Style.Configs.html#set_lines/5"},{"type":"module","title":"Styler.Style.Defs","doc":"Styles function heads so that they're as small as possible.\n\nThe goal is that a function head fits on a single line.\n\nThis isn't a Credo issue, and the formatter is fine with either approach. But Styler has opinions!\n\nEx:\n\nThis long declaration\n\n    def foo(%{\n      bar: baz\n    }) do\n      ...\n    end\n\nBecomes\n\n    def foo(%{bar: baz}) do\n      ...\n    end","ref":"Styler.Style.Defs.html"},{"type":"function","title":"Styler.Style.Defs.run/2","doc":"","ref":"Styler.Style.Defs.html#run/2"},{"type":"module","title":"Styler.Style.Deprecations","doc":"Transformations to soft or hard deprecations introduced on newer Elixir releases","ref":"Styler.Style.Deprecations.html"},{"type":"function","title":"Styler.Style.Deprecations.run/2","doc":"","ref":"Styler.Style.Deprecations.html#run/2"},{"type":"module","title":"Styler.Style.ModuleDirectives","doc":"Styles up module directives!\n\nThis Style will expand multi-aliases/requires/imports/use and sort the directive within its groups (except `use`s, which cannot be sorted)\nIt also adds a blank line after each directive group.","ref":"Styler.Style.ModuleDirectives.html"},{"type":"module","title":"Credo rules - Styler.Style.ModuleDirectives","doc":"Rewrites for the following Credo rules:\n\n  * `Credo.Check.Consistency.MultiAliasImportRequireUse` (force expansion)\n  * `Credo.Check.Readability.AliasOrder` (we sort `__MODULE__`, which credo doesn't)\n  * `Credo.Check.Readability.ModuleDoc` (adds `@moduledoc false` if missing. includes `*.exs` files)\n  * `Credo.Check.Readability.MultiAlias`\n  * `Credo.Check.Readability.StrictModuleLayout` (see section below for details)\n  * `Credo.Check.Readability.UnnecessaryAliasExpansion`\n  * `Credo.Check.Design.AliasUsage`","ref":"Styler.Style.ModuleDirectives.html#module-credo-rules"},{"type":"module","title":"Breakages - Styler.Style.ModuleDirectives","doc":"**This can break your code.**\n\n#","ref":"Styler.Style.ModuleDirectives.html#module-breakages"},{"type":"module","title":"Strict Layout - Styler.Style.ModuleDirectives","doc":"Modules directives are sorted into the following order:\n\n  * `@shortdoc`\n  * `@moduledoc`\n  * `@behaviour`\n  * `use`\n  * `import`\n  * `alias`\n  * `require`\n  * everything else (unchanged)\n\nIf any of the sorted directives had a dependency on code that is now below it, your code will fail to compile after being styled.\n\nFor instance, the following will be broken because the module attribute definition will\nbe moved below the `use` clause, meaning `@pi` is undefined when invoked.\n\n  ```elixir\n  # before\n  defmodule Approximation do\n    @pi 3.14\n    use Math, pi: @pi\n  end\n\n  # after\n  defmodule Approximation do\n    @moduledoc false\n    use Math, pi: @pi\n    @pi 3.14\n  end\n  ```\n\nFor now, it's up to you to come up with a fix for this issue. Sorry!\n\n#","ref":"Styler.Style.ModuleDirectives.html#module-strict-layout"},{"type":"module","title":"Strict Layout: interwoven conflicting aliases - Styler.Style.ModuleDirectives","doc":"Ideally no one writes code like this as it's hard for our human brains to notice the context switching!\nStill, it's a possible source of breakages in Styler.\n\n\n  alias Foo.Bar\n  Bar.Baz.bop()\n\n  alias Baz.Bar\n  Bar.Baz.bop()\n\n  # becomes\n\n  alias Baz.Bar\n  alias Baz.Bar.Baz\n  alias Foo.Bar\n  Baz.bop() # was Foo.Bar.Baz, is now Baz.Bar.Baz\n  Baz.bop()","ref":"Styler.Style.ModuleDirectives.html#module-strict-layout-interwoven-conflicting-aliases"},{"type":"function","title":"Styler.Style.ModuleDirectives.run/2","doc":"","ref":"Styler.Style.ModuleDirectives.html#run/2"},{"type":"module","title":"Styler.Style.Pipes","doc":"Styles pipes! In particular, don't make pipe chains of only one pipe, and some persnickety pipe chain start stuff.\n\nRewrites for the following Credo rules:\n\n  * Credo.Check.Readability.BlockPipe\n  * Credo.Check.Readability.OneArityFunctionInPipe\n  * Credo.Check.Readability.PipeIntoAnonymousFunctions\n  * Credo.Check.Readability.SinglePipe\n  * Credo.Check.Refactor.FilterCount\n  * Credo.Check.Refactor.MapInto\n  * Credo.Check.Refactor.MapJoin\n  * Credo.Check.Refactor.PipeChainStart, excluded_functions: [\"from\"]","ref":"Styler.Style.Pipes.html"},{"type":"function","title":"Styler.Style.Pipes.run/2","doc":"","ref":"Styler.Style.Pipes.html#run/2"},{"type":"module","title":"Styler.Style.SingleNode","doc":"Simple 1-1 rewrites all crammed into one module to make for more efficient traversals\n\nCredo Rules addressed:\n\n* Credo.Check.Consistency.ParameterPatternMatching\n* Credo.Check.Readability.LargeNumbers\n* Credo.Check.Readability.ParenthesesOnZeroArityDefs\n* Credo.Check.Readability.PreferImplicitTry\n* Credo.Check.Readability.StringSigils\n* Credo.Check.Readability.WithSingleClause\n* Credo.Check.Refactor.CaseTrivialMatches\n* Credo.Check.Refactor.CondStatements\n* Credo.Check.Refactor.RedundantWithClauseResult\n* Credo.Check.Refactor.WithClauses","ref":"Styler.Style.SingleNode.html"},{"type":"function","title":"Styler.Style.SingleNode.run/2","doc":"","ref":"Styler.Style.SingleNode.html#run/2"},{"type":"module","title":"Styler.Zipper","doc":"Implements a Zipper for the Elixir AST based on Gérard Huet [Functional pearl: the\nzipper](https://www.st.cs.uni-saarland.de/edu/seminare/2005/advanced-fp/docs/huet-zipper.pdf) paper and\nClojure's `clojure.zip` API.\n\nA zipper is a data structure that represents a location in a tree from the\nperspective of the current node, also called *focus*. It is represented by a\n2-tuple where the first element is the focus and the second element is the\nmetadata/context. The metadata is `nil` when the focus is the topmost node","ref":"Styler.Zipper.html"},{"type":"function","title":"Styler.Zipper.any?/2","doc":"Traverses `zipper`, returning true when `fun.(Zipper.node(zipper))` is truthy, or false otherwise","ref":"Styler.Zipper.html#any?/2"},{"type":"function","title":"Styler.Zipper.append_child/2","doc":"Inserts the item as the rightmost child of the node at this zipper,\nwithout moving.","ref":"Styler.Zipper.html#append_child/2"},{"type":"function","title":"Styler.Zipper.children/1","doc":"Returns a list of children of the node.","ref":"Styler.Zipper.html#children/1"},{"type":"function","title":"Styler.Zipper.down/1","doc":"Returns the zipper of the leftmost child of the node at this zipper, or\nnil if no there's no children.","ref":"Styler.Zipper.html#down/1"},{"type":"function","title":"Styler.Zipper.find/3","doc":"Returns a zipper to the node that satisfies the predicate function, or `nil`\nif none is found.\n\nThe optional second parameters specifies the `direction`, defaults to\n`:next`.","ref":"Styler.Zipper.html#find/3"},{"type":"function","title":"Styler.Zipper.insert_child/2","doc":"Inserts the item as the leftmost child of the node at this zipper,\nwithout moving.","ref":"Styler.Zipper.html#insert_child/2"},{"type":"function","title":"Styler.Zipper.insert_left/2","doc":"Inserts the item as the left sibling of the node at this zipper, without\nmoving. Raises an `ArgumentError` when attempting to insert a sibling at the\ntop level.","ref":"Styler.Zipper.html#insert_left/2"},{"type":"function","title":"Styler.Zipper.insert_right/2","doc":"Inserts the item as the right sibling of the node at this zipper, without\nmoving. Raises an `ArgumentError` when attempting to insert a sibling at the\ntop level.","ref":"Styler.Zipper.html#insert_right/2"},{"type":"function","title":"Styler.Zipper.insert_siblings/2","doc":"Inserts many siblings to the right.\n\nEquivalent to\n\n    Enum.reduce(siblings, zipper, &Zipper.insert_right(&2, &1))","ref":"Styler.Zipper.html#insert_siblings/2"},{"type":"function","title":"Styler.Zipper.left/1","doc":"Returns the zipper of the left sibling of the node at this zipper, or nil.","ref":"Styler.Zipper.html#left/1"},{"type":"function","title":"Styler.Zipper.leftmost/1","doc":"Returns the leftmost sibling of the node at this zipper, or itself.","ref":"Styler.Zipper.html#leftmost/1"},{"type":"function","title":"Styler.Zipper.next/1","doc":"Returns the following zipper in depth-first pre-order.","ref":"Styler.Zipper.html#next/1"},{"type":"function","title":"Styler.Zipper.node/1","doc":"Returns the node at the zipper.","ref":"Styler.Zipper.html#node/1"},{"type":"function","title":"Styler.Zipper.prepend_siblings/2","doc":"Inserts many siblings to the left.\n\nEquivalent to\n\n    Enum.reduce(siblings, zipper, &Zipper.insert_left(&2, &1))","ref":"Styler.Zipper.html#prepend_siblings/2"},{"type":"function","title":"Styler.Zipper.prev/1","doc":"Returns the previous zipper in depth-first pre-order.\nReturns nil if the tree is already at the top.","ref":"Styler.Zipper.html#prev/1"},{"type":"function","title":"Styler.Zipper.remove/1","doc":"Removes the node at the zipper, returning the zipper that would have preceded\nit in a depth-first walk.","ref":"Styler.Zipper.html#remove/1"},{"type":"function","title":"Styler.Zipper.replace/2","doc":"Replaces the current node in the zipper with a new node.","ref":"Styler.Zipper.html#replace/2"},{"type":"function","title":"Styler.Zipper.replace_children/2","doc":"Returns the zipper with the current children of the node replaced with `children`","ref":"Styler.Zipper.html#replace_children/2"},{"type":"function","title":"Styler.Zipper.right/1","doc":"Returns the zipper of the right sibling of the node at this zipper, or nil.","ref":"Styler.Zipper.html#right/1"},{"type":"function","title":"Styler.Zipper.rightmost/1","doc":"Returns the rightmost sibling of the node at this zipper, or itself.","ref":"Styler.Zipper.html#rightmost/1"},{"type":"function","title":"Styler.Zipper.root/1","doc":"Walks the zipper all the way up and returns the root node.","ref":"Styler.Zipper.html#root/1"},{"type":"function","title":"Styler.Zipper.skip/2","doc":"Returns the zipper of the right sibling of the node at this zipper, or the\nnext zipper when no right sibling is available.\n\nThis allows to skip subtrees while traversing the siblings of a node.\n\nThe optional second parameters specifies the `direction`, defaults to\n`:next`.\n\nIf no right/left sibling is available, this function returns the same value as\n`next/1`/`prev/1`.\n\nThe function `skip/1` behaves like the `:skip` in `traverse_while/2` and\n`traverse_while/3`.","ref":"Styler.Zipper.html#skip/2"},{"type":"function","title":"Styler.Zipper.top/1","doc":"Walks the zipper all the way up and returns the top zipper.","ref":"Styler.Zipper.html#top/1"},{"type":"function","title":"Styler.Zipper.traverse/2","doc":"Traverses the tree in depth-first pre-order calling the given function for\neach node.\n\nIf the zipper is not at the top, just the subtree will be traversed.\n\nThe function must return a zipper.","ref":"Styler.Zipper.html#traverse/2"},{"type":"function","title":"Styler.Zipper.traverse/3","doc":"Traverses the tree in depth-first pre-order calling the given function for\neach node with an accumulator.\n\nIf the zipper is not at the top, just the subtree will be traversed.","ref":"Styler.Zipper.html#traverse/3"},{"type":"function","title":"Styler.Zipper.traverse_while/2","doc":"Traverses the tree in depth-first pre-order calling the given function for\neach node.\n\nThe traversing will continue if the function returns `{:cont, zipper}`,\nskipped for `{:skip, zipper}` and halted for `{:halt, zipper}`\n\nIf the zipper is not at the top, just the subtree will be traversed.\n\nThe function must return a zipper.","ref":"Styler.Zipper.html#traverse_while/2"},{"type":"function","title":"Styler.Zipper.traverse_while/3","doc":"Traverses the tree in depth-first pre-order calling the given function for\neach node with an accumulator.\n\nThe traversing will continue if the function returns `{:cont, zipper, acc}`,\nskipped for `{:skip, zipper, acc}` and halted for `{:halt, zipper, acc}`\n\nIf the zipper is not at the top, just the subtree will be traversed.","ref":"Styler.Zipper.html#traverse_while/3"},{"type":"function","title":"Styler.Zipper.up/1","doc":"Returns the zipper of the parent of the node at this zipper, or nil if at the\ntop.","ref":"Styler.Zipper.html#up/1"},{"type":"function","title":"Styler.Zipper.update/2","doc":"Replaces the current node in the zipper with the result of applying `fun` to\nthe node.","ref":"Styler.Zipper.html#update/2"},{"type":"function","title":"Styler.Zipper.zip/1","doc":"Creates a zipper from a tree node.","ref":"Styler.Zipper.html#zip/1"},{"type":"type","title":"Styler.Zipper.command/0","doc":"","ref":"Styler.Zipper.html#t:command/0"},{"type":"opaque","title":"Styler.Zipper.path/0","doc":"","ref":"Styler.Zipper.html#t:path/0"},{"type":"type","title":"Styler.Zipper.t/0","doc":"","ref":"Styler.Zipper.html#t:t/0"},{"type":"type","title":"Styler.Zipper.tree/0","doc":"","ref":"Styler.Zipper.html#t:tree/0"},{"type":"type","title":"Styler.Zipper.zipper/0","doc":"","ref":"Styler.Zipper.html#t:zipper/0"},{"type":"exception","title":"Styler.StyleError","doc":"Wraps errors raised by Styles during tree traversal.","ref":"Styler.StyleError.html"},{"type":"function","title":"Styler.StyleError.message/1","doc":"","ref":"Styler.StyleError.html#message/1"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"main - Changelog","doc":"","ref":"changelog.html#main"},{"type":"extras","title":"1.0.0-alpha.0 - Changelog","doc":"An alpha! You probably want to hold off for the beta, but feel free to take this for a spin and give feedback!\n\nDocumentation to come.\n\n#","ref":"changelog.html#1-0-0-alpha-0"},{"type":"extras","title":"Improvements - Changelog","doc":"##","ref":"changelog.html#improvements"},{"type":"extras","title":"Alias Lifting - Changelog","doc":"Along the lines of `Credo.Check.Design.AliasUsage`, Styler now \"lifts\" deeply nested aliases (depth >= 3, ala `A.B.C....`) that are used more than once.\n\nPut plainly, this code:\n\n```elixir\ndefmodule A do\n  def lift_me() do\n    A.B.C.foo()\n    A.B.C.baz()\n  end\nend\n```\n\nwill become\n\n```elixir\ndefmodule A do\n  @moduledoc false\n  alias A.B.C\n\n  def lift_me do\n    C.foo()\n    C.baz()\n  end\nend\n```\n\nTo exclude modules ending in `.Foo` from being lifted, add `styler: [alias_lifting_exclude: [Foo]]` to your `.formatter.exs`\n\n##","ref":"changelog.html#alias-lifting"},{"type":"extras","title":"Mix Config File Organization - Changelog","doc":"Styler now organizes `Mix.Config.config/2,3` stanzas according to erlang term sorting. This helps manage large configuration files, removing the \"where should I put this\" burden from developers AND helping find duplicated configuration stanzas.\n\nSee the moduledoc for `Styler.Style.Configs` for more.\n\n##","ref":"changelog.html#mix-config-file-organization"},{"type":"extras","title":"Other Improvements - Changelog","doc":"* `if`/`unless`: invert if and unless with `!=` or `!==`, like we do for `!` and `not` #132\n* `@derive`: move `@derive` before `defstruct|schema|embedded_schema` declarations (fixes compiler warning!) #134\n* strings: rewrite double-quoted strings to use `~s` when there's 4+ escaped double-quotes\n  (`\"\\\"\\\"\\\"\\\"\"` -> `~s(\"\"\"\")`) (`Credo.Check.Readability.StringSigils`) #146\n* `Map.drop(foo, [single_key])` => `Map.delete(foo, single_key)` #161 (also in pipes)\n* `Keyword.drop(foo, [single_key])` => `Keyword.delete(foo, single_key)` #161 (also in pipes)\n* `lhs |> Enum.reverse() |> Kernel.++(enum)` => `lhs |> Enum.reverse(enum)`\n\n#","ref":"changelog.html#other-improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* `alias`: expands aliases when moving an alias after another directive that relied on it (#137)\n* module directives: various fixes for unreported obscure crashes\n* pipes: fix a comment-shifting scenario when unpiping\n* `Timex.now/1` will no longer be rewritten to `DateTime.now!/1` due to Timex accepting a wider domain of \"timezones\" than the stdlib (#145, h/t @ivymarkwell)\n* `with`: skip nodes which (unexpectedly) do not contain a `do` body (#158, h/t @DavidB59)\n* `then(&fun/1)`: fix false positives on arithmetic `&1 + x / 1` (#164, h/t @aenglisc)\n\n#","ref":"changelog.html#fixes"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"* drop support for elixir `1.14`\n* sorting configs for the first time can change your configuration; see `Styler.Style.Configs` moduledoc for more","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"v0.11.9 - Changelog","doc":"#","ref":"changelog.html#v0-11-9"},{"type":"extras","title":"Improvements - Changelog","doc":"* pipes: check for `Stream.foo` equivalents to `Enum.foo` in a few more cases\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* pipes: `|> then(&(&1 op y))` rewrites with `|> Kernel.op(y)` as long as the operator is defined in `Kernel`; skips the rewrite otherwise (h/t @kerryb for the report & @saveman71 for the fix)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.11.8 - Changelog","doc":"Two releases in one day!? @koudelka made too good a point about `Map.new` not being special...\n\n#","ref":"changelog.html#v0-11-8"},{"type":"extras","title":"Improvements - Changelog","doc":"* pipes: treat `MapSet.new` and `Keyword.new` the same way we do `Map.new` (h/t @koudelka)\n* pipes: treat `Stream.map` the same as `Enum.map` when piped `|> Enum.into`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.11.7 - Changelog","doc":"#","ref":"changelog.html#v0-11-7"},{"type":"extras","title":"Improvements - Changelog","doc":"* deprecations: `~R` -> `~r`, `Date.range/2` -> `Date.range/3` with decreasing dates (h/t @milmazz)\n* if: rewrite `if not x, do: y` => `unless x, do: y`\n* pipes: `|> Enum.map(foo) |> Map.new()` => `|> Map.new(foo)`\n* pipes: remove unnecessary `then/2` on named function captures: `|> then(&foo/1)` => `|> foo()`, `|> then(&foo(&1, ...))` => `|> foo(...)` (thanks to @tfiedlerdejanze for the idea + impl!)","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.11.6 - Changelog","doc":"#","ref":"changelog.html#v0-11-6"},{"type":"extras","title":"Fixes - Changelog","doc":"* directives: maintain order of module compilation callbacks (`@before_compile` etc) relative to `use` statements (Closes #120, h/t @frankdugan3)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.11.5 - Changelog","doc":"#","ref":"changelog.html#v0-11-5"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix parsing ranges with non-trivial integer bounds like `x..y` (Closes #119, h/t @maennchen)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.11.4 - Changelog","doc":"#","ref":"changelog.html#v0-11-4"},{"type":"extras","title":"Improvements - Changelog","doc":"Shoutout @milmazz for all the deprecation work below =)\n\n* Deprecations: Rewrite 1.16 Deprecations (h/t @milmazz for all the work here)\n  * add `//1` step to `Enum.slice/2|String.slice/2` with decreasing ranges\n  * `File.stream!(file, options, line_or_bytes)` => `File.stream!(file, line_or_bytes, options)`\n* Deprecations `Path.safe_relative_to/2` => `Path.safe_relative/2`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.11.3 - Changelog","doc":"#","ref":"changelog.html#v0-11-3"},{"type":"extras","title":"Fixes - Changelog","doc":"* directives: fix infinite loop when encountering `@spec import(...) :: ...` (Closes #115, h/t @kerryb)\n* `with`: fix deletion of arrow-less `with` statements within function invocations","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.11.2 - Changelog","doc":"#","ref":"changelog.html#v0-11-2"},{"type":"extras","title":"Fixes - Changelog","doc":"* `pipes`: fix unpiping do-blocks into variables when the parent expression is a function invocation\n    like `a(if x do y end |> z(), b)` (Closes #114, h/t @wkirschbaum)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.11.1 - Changelog","doc":"#","ref":"changelog.html#v0-11-1"},{"type":"extras","title":"Fixes - Changelog","doc":"* `with`: fix `with` replacement when it's the only child of a `do` or `->` block (Closes #107, h/t @kerryb -- turns out those edge cases _did_ exist in the wild!)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.11.0 - Changelog","doc":"#","ref":"changelog.html#v0-11-0"},{"type":"extras","title":"Improvements - Changelog","doc":"##","ref":"changelog.html#improvements"},{"type":"extras","title":"Comments - Changelog","doc":"Styler will no longer make comments jump around in any situation, and will move comments with the appropriate node in all cases but module directive rearrangement (where they'll just be left behind - sorry! we're still working on it).\n\n* Keep comments in logical places when rewriting if/unless/cond/with (#79, #97, #101, #103)\n\n##","ref":"changelog.html#comments"},{"type":"extras","title":"With Statements - Changelog","doc":"This release has a slew of improvements for `with` statements. It's not surprising that there's lots of style rules for `with` given that just about any `case`, `if`, or even `cond do` could also be expressed as a `with`. They're very powerful! And with great power...\n\n* style trivial pattern matches ala `lhs <- rhs` to `lhs = rhs` (#86)\n* style `_ <- rhs` to `rhs`\n* style keyword `, do: ` to `do end` rather than wrapping multiple statements in parens\n* style statements all the way to `if` statements when appropriate (#100)\n\n##","ref":"changelog.html#with-statements"},{"type":"extras","title":"Other - Changelog","doc":"* Rewrite `{Map|Keyword}.merge(single_key: value)` to use `put/3` instead (#96)\n\n#","ref":"changelog.html#other"},{"type":"extras","title":"Fixes - Changelog","doc":"* `with`: various edge cases we can only hope no one's encountered and thus never reported","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.10.5 - Changelog","doc":"After being bitten by two of them in a row, Styler's test suite now makes sure that there are no\nidempotency bugs as part of its tests.\n\nIn short, we now have `assert style(x) == style(style(x))` as part of every test. Sorry for not thinking to include this before :)\n\n#","ref":"changelog.html#v0-10-5"},{"type":"extras","title":"Fixes - Changelog","doc":"* alias: fix single-module alias deletion newlines bug\n* comments: ensure all generated nodes always include line meta (#101)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.10.4 - Changelog","doc":"#","ref":"changelog.html#v0-10-4"},{"type":"extras","title":"Improvements - Changelog","doc":"* alias: delete noop single-module aliases (`alias Foo`, #87, h/t @mgieger)\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* pipes: unnest all pipe starts in one pass (`f(g(h(x))) |> j()` => `x |> h() |> g() |> f() |> j()`, #94, h/t @tomjschuster)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.10.3 - Changelog","doc":"#","ref":"changelog.html#v0-10-3"},{"type":"extras","title":"Improvements - Changelog","doc":"* charlists: leave charlist rewriting to elixir's formatter on elixir >= 1.15\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* charlists: rewrite empty charlist to use sigil (`''` => `~c\"\"`)\n* pipes: don't blow up extracting fully-qualified macros (`Foo.bar do end |> foo()`, #91, h/t @NikitaNaumenko)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.10.2 - Changelog","doc":"#","ref":"changelog.html#v0-10-2"},{"type":"extras","title":"Improvements - Changelog","doc":"* `with`: remove identity singleton else clause (eg `else {:error, e} -> {:error, e} end`, `else error -> error end`)","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.10.1 - Changelog","doc":"#","ref":"changelog.html#v0-10-1"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix function head shrink-failures causing comments to jump into blocks (Closes #67, h/t @APB9785)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.10.0 - Changelog","doc":"#","ref":"changelog.html#v0-10-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* hoist all block-starts to pipes to their own variables (makes styler play better with piped macros)\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix pipes starting with a macro do-block creating invalid ast (#83, h/t @mhanberg)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.9.7 - Changelog","doc":"#","ref":"changelog.html#v0-9-7"},{"type":"extras","title":"Fixes - Changelog","doc":"* rewrite pipes starting with `quote` blocks like we do with `case|if|cond|with` blocks (#82, h/t @SteffenDE)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.9.6 - Changelog","doc":"#","ref":"changelog.html#v0-9-6"},{"type":"extras","title":"Breaking Change - Changelog","doc":"* removed `mix style` task","ref":"changelog.html#breaking-change"},{"type":"extras","title":"v0.9.5 - Changelog","doc":"#","ref":"changelog.html#v0-9-5"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix mistaking `Timex.now/1` in a pipe for `Timex.now/0` (#66, h/t @sabiwara)\n\n#","ref":"changelog.html#fixes"},{"type":"extras","title":"Removed style - Changelog","doc":"* stop rewriting `Timex.today/0` given that we allow `Timex.today/1` -- too inconsistent.","ref":"changelog.html#removed-style"},{"type":"extras","title":"v0.9.4 - Changelog","doc":"#","ref":"changelog.html#v0-9-4"},{"type":"extras","title":"Improvements - Changelog","doc":"* `if` statements: drop `else` clauses whose body is simply `nil`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.9.3 - Changelog","doc":"#","ref":"changelog.html#v0-9-3"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix `unless a do b else c end` rewrites to `if` not flopping do/else bodies! (#77, h/t @jcowgar)\n* fix pipes styling ranges with steps (`a..b//c`) incorrectly (#76, h/t @cschmatzler)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.9.2 - Changelog","doc":"#","ref":"changelog.html#v0-9-2"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix exception styling module attributes named `@def` (we confused them with real `def`s, whoops!) (#75, h/t @randycoulman)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.9.1 - Changelog","doc":"the boolean blocks edition!\n\n#","ref":"changelog.html#v0-9-1"},{"type":"extras","title":"Improvements - Changelog","doc":"* auto-fix `Credo.Check.Refactor.CondStatements` (detects any truthy atom, not just `true`)\n* if/unless rewrites:\n  - `Credo.Check.Refactor.NegatedConditionsWithElse`\n  - `Credo.Check.Refactor.NegatedConditionsInUnless`\n  - `Credo.Check.Refactor.UnlessWithElse`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.9.0 - Changelog","doc":"the with statement edition!\n\n#","ref":"changelog.html#v0-9-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* Added right-hand-pattern-matching rewrites to `for` and `with` left arrow expressions `<-`\n  (ex: `with map = %{} <- foo()` => `with %{} = map <- foo`)\n* `with` statement rewrites, solving the following credo rules\n  * `Credo.Check.Readability.WithSingleClause`\n  * `Credo.Check.Refactor.RedundantWithClauseResult`\n  * `Credo.Check.Refactor.WithClauses`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.8.5 - Changelog","doc":"#","ref":"changelog.html#v0-8-5"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fixed exception when encountering non-arrowed case statements ala `case foo, do: unquote(quoted)` (#69, h/t @brettinternet, nice)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.8.4 - Changelog","doc":"#","ref":"changelog.html#v0-8-4"},{"type":"extras","title":"Fixes - Changelog","doc":"* Timex related fixes (#66):\n  * Rewrite `Timex.now/1` to `DateTime.now!/1` instead of `DateTime.utc_now/1`\n  * Only rewrite `Timex.today/0`, don't change `Timex.today/1`","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.8.3 - Changelog","doc":"#","ref":"changelog.html#v0-8-3"},{"type":"extras","title":"Improvements - Changelog","doc":"* DateTime rewrites (#62, ht @milmazz)\n  * `DateTime.compare` => `DateTime.{before/after}` (elixir >= 1.15)\n  * `Timex.now` => `DateTime.utc_now`\n  * `Timex.today` => `Date.utc_today`\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* Pipes: add  `!=`, `!==`, `===`, `and`, and `or` to list of valid infix operators (#64)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.8.2 - Changelog","doc":"#","ref":"changelog.html#v0-8-2"},{"type":"extras","title":"Fixes - Changelog","doc":"* Pipes always de-sugars keyword lists when unpiping them (#60)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.8.1 - Changelog","doc":"#","ref":"changelog.html#v0-8-1"},{"type":"extras","title":"Fixes - Changelog","doc":"* ModuleDirectives doesn't mistake variables for directives (#57, h/t @leandrocp)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.8.0 - Changelog","doc":"#","ref":"changelog.html#v0-8-0"},{"type":"extras","title":"Improvements (Bug Fix!?) - Changelog","doc":"* ModuleDirectives no longer throws comments around a file when hoisting directives up (#53)","ref":"changelog.html#improvements-bug-fix"},{"type":"extras","title":"v0.7.14 - Changelog","doc":"#","ref":"changelog.html#v0-7-14"},{"type":"extras","title":"Improvements - Changelog","doc":"* rewrite `Logger.warn/1,2` to `Logger.warning/1,2` due to Elixir 1.15 deprecation","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.7.13 - Changelog","doc":"#","ref":"changelog.html#v0-7-13"},{"type":"extras","title":"Fixes - Changelog","doc":"* don't unpipe single-piped `unquote` expressions (h/t @elliottneilclark)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.7.12 - Changelog","doc":"#","ref":"changelog.html#v0-7-12"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix 0-arity paren removal on metaprogramming creating uncompilable code (h/t @simonprev)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.7.11 - Changelog","doc":"#","ref":"changelog.html#v0-7-11"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix crash from `mix style` running plugins as part of formatting (no longer runs formatter plugins)\n\n#","ref":"changelog.html#fixes"},{"type":"extras","title":"Improvements - Changelog","doc":"* single-quote charlists are rewritten to use the `~c` sigil (`'foo'` -> `~c'foo'`) (h/t @fhunleth)\n* `mix style` warns the user that Styler is primarily meant to be used as a plugin","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.7.10 - Changelog","doc":"#","ref":"changelog.html#v0-7-10"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix crash when encountering single-quote charlists (h/t @fhunleth)\n\n#","ref":"changelog.html#fixes"},{"type":"extras","title":"Improvements - Changelog","doc":"* single-quote charlists are rewritten to use the `~c` sigil (`'foo'` -> `~c'foo'`)\n* when encountering `_ = bar ->`, replace it with `bar ->`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.7.9 - Changelog","doc":"#","ref":"changelog.html#v0-7-9"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix a toggle state resulting from (ahem, nonsense) code like `_ = bar ->` encountering ParameterPatternMatching style","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.7.8 - Changelog","doc":"#","ref":"changelog.html#v0-7-8"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix crash trying to remove 0-arity parens from metaprogramming ala `def unquote(foo)()`","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.7.7 - Changelog","doc":"#","ref":"changelog.html#v0-7-7"},{"type":"extras","title":"Improvements - Changelog","doc":"* Rewrite `Enum.into/2,3` into `Map.new/1,2` when the collectable is `%{}` or `Map.new/0`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.7.6 - Changelog","doc":"#","ref":"changelog.html#v0-7-6"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix crash when single pipe had inner defs (h/t [@michallepicki](https://github.com/adobe/elixir-styler/issues/39))","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.7.5 - Changelog","doc":"#","ref":"changelog.html#v0-7-5"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix bug from `ParameterPatternMatching` implementation that re-ordered pattern matching in `cond do` `->` clauses","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.7.4 - Changelog","doc":"#","ref":"changelog.html#v0-7-4"},{"type":"extras","title":"Features - Changelog","doc":"* Implement `Credo.Check.Readability.PreferImplicitTry`\n* Implement `Credo.Check.Consistency.ParameterPatternMatching` for `def|defp|fn|case`","ref":"changelog.html#features"},{"type":"extras","title":"v0.7.3 - Changelog","doc":"#","ref":"changelog.html#v0-7-3"},{"type":"extras","title":"Features - Changelog","doc":"* Remove parens from 0-arity function definitions (`Credo.Check.Readability.ParenthesesOnZeroArityDefs`)","ref":"changelog.html#features"},{"type":"extras","title":"v0.7.2 - Changelog","doc":"#","ref":"changelog.html#v0-7-2"},{"type":"extras","title":"Features - Changelog","doc":"* Rewrite `case ... true -> ...; _ -> ...` to `if` statements as well","ref":"changelog.html#features"},{"type":"extras","title":"v0.7.1 - Changelog","doc":"#","ref":"changelog.html#v0-7-1"},{"type":"extras","title":"Features - Changelog","doc":"* Rewrite `case ... true / else ->` to be `if` statements","ref":"changelog.html#features"},{"type":"extras","title":"v0.7.0 - Changelog","doc":"#","ref":"changelog.html#v0-7-0"},{"type":"extras","title":"Features - Changelog","doc":"* `Styler.Style.Simple`:\n    * Optimize `Enum.reverse(foo) ++ bar` to `Enum.reverse(foo, bar)`\n* `Styler.Style.Pipes`:\n    * Rewrite `|> (& ...).()` to `|> then(& ...)` (`Credo.Check.Readability.PipeIntoAnonymousFunctions`)\n    * Add parens to 1-arity pipe functions (`Credo.Check.Readability.OneArityFunctionInPipe`)\n    * Optimize `a |> Enum.reverse() |> Enum.concat(enum)` to `Enum.reverse(a, enum)`","ref":"changelog.html#features"},{"type":"extras","title":"v0.6.1 - Changelog","doc":"#","ref":"changelog.html#v0-6-1"},{"type":"extras","title":"Improvements - Changelog","doc":"* Better error handling: `mix format` will still format files if a style fails\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* `mix style`: only run on `.ex` and `.exs` files\n* `ModuleDirectives`: now expands `alias __MODULE__.{A, B}` (h/t [@adriankumpf](https://github.com/adriankumpf))","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.6.0 - Changelog","doc":"#","ref":"changelog.html#v0-6-0"},{"type":"extras","title":"Features - Changelog","doc":"* `mix style`: brought back to life for folks who want to incrementally introduce Styler\n\n#","ref":"changelog.html#features"},{"type":"extras","title":"Fixes - Changelog","doc":"* `Styler.Style.Pipes`:\n   * include `x in y` and `^foo` (for ecto) as a valid pipe starts\n   * work even harder to keep rewrites on one line","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.5.2 - Changelog","doc":"#","ref":"changelog.html#v0-5-2"},{"type":"extras","title":"Fixes - Changelog","doc":"* `ModuleDirectives`: handle dynamic module names\n* `Pipes`: include `Ecto.Query.from` and `Query.from` as valid pipe starts","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.5.1 - Changelog","doc":"#","ref":"changelog.html#v0-5-1"},{"type":"extras","title":"Improvements - Changelog","doc":"* Sped up styling just a little bit","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.5.0 - Changelog","doc":"#","ref":"changelog.html#v0-5-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* `Styler` now implements `Mix.Task.Format`, meaning it is now an Elixir formatter plugin.\nSee the README for new installation & usage instructions\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Breaking Change! Wooo! - Changelog","doc":"* the `mix style` task has been removed","ref":"changelog.html#breaking-change-wooo"},{"type":"extras","title":"v0.4.1 - Changelog","doc":"#","ref":"changelog.html#v0-4-1"},{"type":"extras","title":"Improvements - Changelog","doc":"* `Pipes` rewrites `|> Enum.into(%{}[, mapper])` and `Enum.into(Map.new()[, mapper])` to `Map.new/1,2` calls","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"#","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* `Pipes` rewrites some two-step processes into one, fixing these credo issues in pipe chains:\n    * `Credo.Check.Refactor.FilterCount`\n    * `Credo.Check.Refactor.MapJoin`\n    * `Credo.Check.Refactor.MapInto`\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* `ModuleDirectives` handles even weirder places to hide your aliases (anonymous functions, in this case)\n* `Pipes` tries even harder to keep single-pipe rewrites of invocations on one line","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.3.1 - Changelog","doc":"#","ref":"changelog.html#v0-3-1"},{"type":"extras","title":"Fixes - Changelog","doc":"* `Pipes`\n    * fixed omission of `==` as a valid pipe start operator (h/t @peake100 for the issue)\n    * fixed rewrite of `a |> b`, where `b` was invoked without parenthesis","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"#","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* Enabled `Defs` style and overhauled it to properly handles comments\n* Optimized and tweaked `ModuleDirectives` style\n    * Now culls newlines between \"groups\" of the same directive\n    * sorts `@behaviour` directives\n    * orders directives within non defmodule contexts (eg, a `def do`) if there's at least one `alias|require|use|import`\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* `Pipes` will try to keep single-pipe rewrites on one line","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"#","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* Added `ModuleDirectives` style\n    * Note that this is potentially destructive in some rare cases. See moduledoc for more.\n    * This supersedes the `Aliases` style, which has been removed.\n* `mix style -` reads and writes to stdin/stdout\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* `Pipes` style is now aware of `unless` blocks","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.1.1 - Changelog","doc":"#","ref":"changelog.html#v0-1-1"},{"type":"extras","title":"Improvements - Changelog","doc":"* Lots of README tweaking =)\n* Optimized some Zipper operations\n* Added `Simple` style, replacing the following Credo rule:\n    * `Credo.Check.Readability.LargeNumbers`\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* Exceptions while parsing code now appropriately render filename rather than `nofile:xx`\n* Fixed opaque `Zipper.path()` typespec implementation mismatches (thanks @sega-yarkin)\n* Made `ex_doc` dev only, removing it as a dependency for users of Styler","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"#","ref":"changelog.html#v0-1-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* Initial release of Styler\n* Added `Aliases` style, replacing the following Credo rules:\n    * `Credo.Check.Readability.AliasOrder`\n    * `Credo.Check.Readability.MultiAlias`\n    * `Credo.Check.Readability.UnnecessaryAliasExpansion`\n* Added `Pipes` style, replacing the following Credo rules:\n    * `Credo.Check.Readability.BlockPipe`\n    * `Credo.Check.Readability.SinglePipe`\n    * `Credo.Check.Refactor.PipeChainStart`\n* Added `Defs` style (currently disabled by default)","ref":"changelog.html#improvements"},{"type":"extras","title":"Styler","doc":"# Styler\n\nStyler is an Elixir formatter plugin that's combination of `mix format` and `mix credo`, except instead of telling\nyou what's wrong, it just rewrites the code for you to fit its style rules.\n\nYou can learn more about the history, purpose and implementation of Styler from our talk: [Styler: Elixir Style-Guide Enforcer @ GigCity Elixir 2023](https://www.youtube.com/watch?v=6pF8Hl5EuD4)","ref":"readme.html"},{"type":"extras","title":"Installation - Styler","doc":"Add `:styler` as a dependency to your project's `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:styler, \"~> 0.11\", only: [:dev, :test], runtime: false},\n  ]\nend\n```\n@TODO put this somewhere more reasonable\n**Note** Styler's only public API is its usage as a formatter plugin. While you're welcome to play with its internals,\nthey can and will change without that change being reflected in Styler's semantic version.\n\nThen add `Styler` as a plugin to your `.formatter.exs` file\n\n```elixir\n[\n  plugins: [Styler]\n  # optionally: include styler configuration\n  # , styler: [alias_lifting_excludes: []]\n]\n```\n\nAnd that's it! Now when you run `mix format` you'll also get the benefits of Styler's Stylish Stylings.\n\n#","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - Styler","doc":"@TODO document: config for lifting, and why we won't add options other configs\n\nStyler is @adobe's internal Style Guide Enforcer - allowing exceptions to the styles goes against that ethos. Happily, it's open source and thus yours to do with as you will =)","ref":"readme.html#configuration"},{"type":"extras","title":"Features (or as we call them, \"Styles\") - Styler","doc":"@TODO link examples","ref":"readme.html#features-or-as-we-call-them-styles"},{"type":"extras","title":"Styler & Credo - Styler","doc":"@TODO link credo doc","ref":"readme.html#styler-credo"},{"type":"extras","title":"Your first Styling - Styler","doc":"**Speed**: Expect the first run to take some time as `Styler` rewrites violations of styles.\n\nOnce styled the first time, future styling formats shouldn't take noticeably more time.","ref":"readme.html#your-first-styling"},{"type":"extras","title":"Styler can break your code - Styler","doc":"@TODO link troubleshooting\nmention our rewrite of case true false to if and how we're OK with this being _Styler_, not _SemanticallyEquivalentRewriter_.","ref":"readme.html#styler-can-break-your-code"},{"type":"extras","title":"Thanks & Inspiration - Styler","doc":"### [Sourceror](https://github.com/doorgan/sourceror/)\n\nStyler's first incarnation was as one-off scripts to rewrite an internal codebase to allow Credo rules to be turned on.\n\nThese rewrites were entirely powered by the terrific `Sourceror` library.\n\nWhile `Styler` no longer relies on `Sourceror`, we're grateful for its author's help with those scripts, the inspiration\nSourceror provided in showing us what was possible, and the changes to the Elixir AST APIs that it drove.\n\nStyler's [AST-Zipper](`m:Styler.Zipper`) implementation in this project was forked from Sourceror. Zipper has been a crucial\npart of our ability to ergonomically zip around (heh) Elixir AST.\n\n### [Credo](https://github.com/rrrene/credo/)\n\nWe never would've bothered trying to rewrite our codebase if we didn't have Credo rules we wanted to apply.\n\nCredo's tests and implementations were referenced for implementing Styles that took the work the rest of the way.\n\nThanks to Credo & the Elixir community at large for coalescing around many of these Elixir style credos.","ref":"readme.html#thanks-inspiration"},{"type":"extras","title":"Examples","doc":"# Styles","ref":"styles.html"},{"type":"extras","title":"Simple (Single Node) Styles - Examples","doc":"Function Performance & Readability Optimizations\n\nOptimizing for either performance or readability, probably both!\nThese apply to the piped versions as well\n\n\n#","ref":"styles.html#simple-single-node-styles"},{"type":"extras","title":"Strings to Sigils - Examples","doc":"Rewrites strings with 4 or more escaped quotes to string sigils with an alternative delimiter.\nThe delimiter will be one of `\" ( { | [ ' < /`, chosen by which would require the fewest escapes, and otherwise preferred in the order listed.\n\n##","ref":"styles.html#strings-to-sigils"},{"type":"extras","title":"Before - Examples","doc":"```elixir\nconn\n|> put_resp_content_type(\"application/json\")\n|> send_resp(403, \"{\\\"errors\\\":[\\\"Not Authorized\\\"]}\")\n|> halt()\n```\n\n##","ref":"styles.html#before"},{"type":"extras","title":"After - Examples","doc":"```elixir\nconn\n|> put_resp_content_type(\"application/json\")\n|> send_resp(403, ~s({\"errors\":[\"Not Authorized\"]})))\n|> halt()\n```\n\n#","ref":"styles.html#after"},{"type":"extras","title":"Large Base 10 Numbers - Examples","doc":"Style base 10 numbers with 5 or more digits to have a `_` every three digits.\nFormatter already does this except it doesn't rewrite \"typos\" like `100_000_0`.\n\nIf you're concerned that this breaks your team's formatting for things like \"cents\" (like \"$100\" being written as `100_00`),\nconsider using a library made for denoting currencies rather than raw elixir integers.\n\n##","ref":"styles.html#large-base-10-numbers"},{"type":"extras","title":"Before - Examples","doc":"```elixir\n10000\n1_0_0_0_0 # Elixir's formatter is fine with this\n-543213\n123456789\n55333.22\n-123456728.0001\n```\n\n##","ref":"styles.html#before"},{"type":"extras","title":"After - Examples","doc":"```elixir\n10_000\n10_000\n-543_213\n123_456_789\n55_333.22\n-123_456_728.0001\n```\n\n### `Enum.into(%{}/Map/Keyword/MapSet.new)` -> `X.new`\n\nWhile these examples use `%{}`, the same behaviour occurs for `Keyword.new()`, `MapSet.new()` and the empty map `%{}`.\n\nThis is an improvement for the reader, who gets a more natural language expression: \"make a new map from a\" vs \"take a and enumerate it into a new map\"\n\n##","ref":"styles.html#after"},{"type":"extras","title":"Before - Examples","doc":"```elixir\nEnum.into(a, %{})\nEnum.into(a, %{}, mapping_function)\n```\n\n##","ref":"styles.html#before"},{"type":"extras","title":"After - Examples","doc":"```elixir\nMap.new(a)\nMap.new(a, mapping_function)\n```\n\n- Enum.into(%{}/Map/Keyword/MapSet.new) -> X.new\n\n#","ref":"styles.html#after"},{"type":"extras","title":"Map/Keyword.merge w/ single key literal -> X.put - Examples","doc":"`Keyword.merge` and `Map.merge` called with a literal map or keyword argument with a single key are rewritten to the\nequivalent `put`, a cognitively simpler function.\n\n##","ref":"styles.html#map-keyword-merge-w-single-key-literal-x-put"},{"type":"extras","title":"Before - Examples","doc":"```elixir\nfoo |> Keyword.merge(%{just_one_key: the_value}) |> bar()\n```\n\n##","ref":"styles.html#before"},{"type":"extras","title":"After - Examples","doc":"```elixir\nfoo |> Keyword.put(:just_one_key, the_value) |> bar()\n```\n\n#","ref":"styles.html#after"},{"type":"extras","title":"Map/Keyword.drop w/ single key -> X.delete - Examples","doc":"In the same vein as the `merge` style above, `[Map|Keyword].drop/2` with a single key to drop are rewritten to use `delete/2`\n\n##","ref":"styles.html#map-keyword-drop-w-single-key-x-delete"},{"type":"extras","title":"Before - Examples","doc":"```elixir\nMap.drop(foo, [key])\n```\n##","ref":"styles.html#before"},{"type":"extras","title":"After - Examples","doc":"```elixir\nMap.delete(foo, key)\n```\n\n### `Enum.reverse(foo) ++ bar -> Enum.reverse(foo, bar)`\n\n`Enum.reverse/2` optimizes a two-step reverse and concatenation into a single step.\n\n##","ref":"styles.html#after"},{"type":"extras","title":"Before - Examples","doc":"```elixir\nEnum.reverse(foo) ++ bar\n\nbaz\n|> Enum.reverse()\n|> Enum.concat(bop)\n\n```\n##","ref":"styles.html#before"},{"type":"extras","title":"After - Examples","doc":"```elixir\nEnum.reverse(foo, bar)\n\nEnum.reverse(baz, bop)\n```\n\n#","ref":"styles.html#after"},{"type":"extras","title":"Timex.now/0 -> DateTime.utc_now/0 - Examples","doc":"Timex certainly has its uses, but knowing what stdlib date/time struct is returned by `now/0` is a bit difficult!\nWe prefer calling the actual function rather than its rename in Timex, helping the reader by being more explicit.\n\n##","ref":"styles.html#timex-now-0-datetime-utc_now-0"},{"type":"extras","title":"Before - Examples","doc":"```elixir\nTimex.now()\n```\n##","ref":"styles.html#before"},{"type":"extras","title":"After - Examples","doc":"```elixir\nDateTime.utc_now()\n```\n\n\n#","ref":"styles.html#after"},{"type":"extras","title":"DateModule.compare(x, y) == :lt/:gt -> DateModule.before?/after? - Examples","doc":"Again, the goal is readability and maintainability. `before?/2` and `after?/2` were implemented long after `compare/2`,\nso it's not unusual that a codebase needs a lot of refactoring to be brought up to date with these new functions.\nThat's where Styler comes in!\n\n##","ref":"styles.html#datemodule-compare-x-y-lt-gt-datemodule-before-after"},{"type":"extras","title":"Before - Examples","doc":"```elixir\nif DateTime.compare(start, end) == :gt,\n  do: :error,\n  else: :ok\n```\n##","ref":"styles.html#before"},{"type":"extras","title":"After - Examples","doc":"```elixir\nif DateTime.after?(start, end),\n  do: :error,\n  else: :ok\n```\n\n#","ref":"styles.html#after"},{"type":"extras","title":"Code Readability - Examples","doc":"- put matches on right\n- `Credo.Check.Readability.PreferImplicitTry`\n\n#","ref":"styles.html#code-readability"},{"type":"extras","title":"Consistency - Examples","doc":"- `def foo()` -> `def foo`\n\n#","ref":"styles.html#consistency"},{"type":"extras","title":"Elixir Deprecation Rewrites - Examples","doc":"1.15+\n\n- Logger.warn -> Logger.warning\n- Path.safe_relative_to/2 => Path.safe_relative/2\n- Enum/String.slice/2 w/ ranges -> explicit steps\n- ~R/my_regex/ -> ~r/my_regex/\n- Date.range/2 -> Date.range/3 when decreasing range\n- IO.read/bin_read -> use `:eof` instead of `:all`\n\n1.16+\n\n- File.stream!(file, options, line_or_bytes) => File.stream!(file, line_or_bytes, options)\n\n#","ref":"styles.html#elixir-deprecation-rewrites"},{"type":"extras","title":"Function Definitions - Examples","doc":"- Shrink multi-line function defs\n- Put assignments on the right","ref":"styles.html#function-definitions"},{"type":"extras","title":"Module Directives (`use`, `import`, `alias`, `require`, ...) - Examples","doc":"","ref":"styles.html#module-directives-use-import-alias-require"},{"type":"extras","title":"Mix Configs - Examples","doc":"Mix Config files have their config stanzas sorted. Similar to the sorting of aliases, this delivers consistency to an otherwise arbitrary world, and can even help catch bugs like configuring the same key multiple times.\n\nA file is considered a config file if\n\n1. its path matches `config/.*\\.exs` or `rel/overlays/.*\\.exs`\n2. the file imports Mix.Config (`import Mix.Config`)\n\nOnce a file is detected as a mix config, its `config/2,3` stanzas are grouped and ordered like so:\n\n- group config stanzas separated by assignments (`x = y`) together\n- sort each group according to erlang term sorting\n- move all existing assignments between the config stanzas to above the stanzas (without changing their ordering)","ref":"styles.html#mix-configs"},{"type":"extras","title":"Control Flow Structures (aka \"Blocks\": `case`, `if`, `unless`, `cond`, `with`) - Examples","doc":"### `case`\n\n- rewrite to `if` for `true/false`, `true/_`, `false/true`\n\n\n### `with`\n\n`with` great power comes a great responsibility. don't use `with` when another (simpler!) \"Control Flow Structure\"\n\n- single statement `with` with `else` clauses is rewritten to `case` (which can be further rewritten to an `if`!)\n- move non `<-` out of the head and into preroll or body\n- fully replace with statement with normal code as\n- drop redundant identity else clause `else: (error -> error)` (also more complex matches, ala `{:error, error} -> {:error, error}`)\n- Credo.Check.Refactor.RedundantWithClauseResult\n\n### `cond`\n- Credo.Check.Refactor.CondStatements\n\n### `if`/`unless`\n\nif/unless often looks to see if the root of the statement is a \"negator\", defined as one of the following operators: `:!, :not, :!=, :!==`. We always try to rewrite if/unless statements to not be negated, using the inverse construct when appropriate (but we'll never write an unless with an `else`)\n\n- repeated negators (`!!`) are removed\n- negated if/unless without an `else` are inverted to unless/if (this is done recursively until 0 or 1 negations remain)\n- `unless` with `else` are inverted to negated `if` statements\n- negated `if` with `else` have their clauses inverted to remove the negation\n- if/unless with `else: nil` is dropped as redundant","ref":"styles.html#control-flow-structures-aka-blocks-case-if-unless-cond-with"},{"type":"extras","title":"Pipe Chains - Examples","doc":"#","ref":"styles.html#pipe-chains"},{"type":"extras","title":"Pipe Start - Examples","doc":"- raw value\n- blocks are extracted to variables\n- ecto's `from` is allowed\n\n#","ref":"styles.html#pipe-start"},{"type":"extras","title":"Piped function rewrites - Examples","doc":"- add parens to function calls `|> fun |>` => `|> fun() |>`\n- remove unnecessary `then/2`: `|> then(&f(&1, ...))` -> `|> f(...)`\n- add `then` when defining anon funs in pipe `|> (& &1).() |>` => `|> |> then(& &1) |>`\n\n#","ref":"styles.html#piped-function-rewrites"},{"type":"extras","title":"Piped function optimizations - Examples","doc":"Two function calls into one! Tries to fit everything on one line when shrinking.\n\n- `lhs |> Enum.reverse() |> Enum.concat(enum)` => `lhs |> Enum.reverse(enum)` (also Kernel.++)\n- `lhs |> Enum.filter(filterer) |> Enum.count()` => `lhs |> Enum.count(count)`\n- `lhs |> Enum.map(mapper) |> Enum.join(joiner)` => `lhs |> Enum.map_join(joiner, mapper)`\n- `lhs |> Enum.map(mapper) |> Enum.into(empty_map)` => `lhs |> Map.new(mapper)`\n- `lhs |> Enum.map(mapper) |> Enum.into(collectable)` => `lhs |> Enum.into(collectable, mapper)`\n- `lhs |> Enum.map(mapper) |> Map.new()` => `lhs |> Map.new(mapper)` mapset & keyword also\n\n#","ref":"styles.html#piped-function-optimizations"},{"type":"extras","title":"Unpiping Single Pipes - Examples","doc":"- notably, optimizations might turn a 2 pipe into a single pipe\n- doesn't unpipe when we're starting w/ quote\n- pretty straight forward i daresay","ref":"styles.html#unpiping-single-pipes"}],"content_type":"text/markdown"}